@@ -6,6 +6,7 @@
 build_tt <- function(x, output = NULL) {
   output <- sanitize_output(output)
 
+
   x <- switch(output,
     html = swap_class(x, "tinytable_bootstrap"),
     latex = swap_class(x, "tinytable_tabularray"),
@@ -17,13 +18,11 @@ build_tt <- function(x, output = NULL) {
 
   x@output <- output
 
-  # groups must increment indices here
-  for (idx in seq_along(x@lazy_group)) {
-    l <- x@lazy_group[[idx]]
-    x@nrow <- x@nrow + length(l$i)
-    if (length(l$j) > 0) {
-      x@nhead <- x@nhead + 1
-    }
+  for (th in x@lazy_theme) {
+    fn <- th[[1]]
+    args <- th[[2]]
+    args[["x"]] <- x
+    x <- do.call(fn, args)
   }
 
   tab <- x@table_dataframe
@@ -66,10 +65,7 @@ build_tt <- function(x, output = NULL) {
 
   # markdown styles need to be applied before creating the table, otherwise there's annoying parsing, etc.
   if (x@output %in% c("markdown", "gfm", "dataframe")) {
-    for (l in x@lazy_style) {
-      l[["x"]] <- x
-      x <- eval(l)
-    }
+    x <- style_eval(x)
   }
 
   # draw the table
@@ -86,20 +82,6 @@ build_tt <- function(x, output = NULL) {
     x <- eval(l)
   }
 
-  if (x@output == "typst") {
-    if (is.null(x@theme[[1]]) || is.function(x@theme[[1]]) || isTRUE(x@theme[[1]] %in% c("default", "striped"))) {
-      # reverse the order of the lines to allow overwriting defaults
-      ls <- x@lazy_style 
-      x <- style_tt(x, i = nrow(x), line = "b", line_width = 0.1)
-      if (x@nhead > 0) {
-        x <- style_tt(x, i = -x@nhead + 1, line = "t", line_width = 0.1)
-        x <- style_tt(x, i = 1, line = "t", line_width = 0.05)
-      } else {
-        x <- style_tt(x, i = 1, line = "t", line_width = 0.1)
-      }
-    }
-  }
-
   if (!x@output %in% c("markdown", "gfm", "dataframe")) {
     for (l in x@lazy_style) {
       l[["x"]] <- x
@@ -110,8 +92,9 @@ build_tt <- function(x, output = NULL) {
     }
   }
 
-  if (x@output == "typst") {
-    x <- style_apply_typst(x)
+  # markdown styles are applied earlier
+  if (!x@output %in% c("markdown", "gfm", "dataframe")) {
+    x <- style_eval(x)
   }
 
   x <- finalize(x)
