@@ -20,8 +20,8 @@ group_tabularray_col <- function(x, j, ihead, ...) {
   out <- strsplit(x@table_string, split = "\\n")[[1]]
 
   header <- rep("", ncol(x))
-  for (n in names(j)) {
-    header[min(j[[n]])] <- n
+  for (idx in seq_along(j)) {
+    header[min(j[[idx]])] <- names(j)[idx]
   }
   header <- paste(header, collapse = " & ")
 
@@ -46,14 +46,16 @@ group_tabularray_col <- function(x, j, ihead, ...) {
 
   for (k in seq_along(j)) {
     z <- min(j[[k]])
+    cs <- max(j[[k]]) - min(j[[k]]) + 1
+    if (cs == 1) cs <- NULL
     args <- list(
       tt_build_now = TRUE,
       x = x,
       i = ihead,
       j = z,
       align = "c",
-      colspan = max(j[[k]]) - min(j[[k]]) + 1)
-    x <- do.call(style_eval, args)
+      colspan = cs)
+    x <- do.call(style_tt, args)
   }
 
   return(x)
@@ -95,6 +97,8 @@ group_tabularray_row <- function(x, i, indent) {
   tab <- c(top, idx$vec, bot)
   tab <- paste(tab, collapse = "\n")
 
+  # colspan for row groups
+  # can't figure out how to use style_tt() here. Maybe build order?
   cellspec <- sprintf(
     "cell{%s}{%s}={%s}{%s},",
     idx$new[is.na(idx$old)] + x@nhead,
@@ -105,12 +109,6 @@ group_tabularray_row <- function(x, i, indent) {
   cellspec <- paste(cellspec, collapse = "")
   tab <- tabularray_insert(tab, content = cellspec, type = "inner")
 
-  # we also want to indent the header
-  i <- idx$new[!is.na(idx$old)] + x@nhead
-  cellspec <- sprintf("cell{%s}{%s}={%s},\n", i, 1, sprintf("preto={\\hspace{%sem}}", indent))
-  cellspec <- paste(cellspec, collapse = "")
-  tab <- tabularray_insert(tab, content = cellspec, type = "inner")
-
   x@table_string <- tab
 
   return(x)
