@@ -24,7 +24,6 @@ group_typst_row <- function(x, i, indent, ...) {
   tab <- strsplit(tab, split = "\\n")[[1]]
   body_min <- utils::head(grep("tinytable cell content after", tab), 1) + 1
   body_max <- utils::head(grep("end table", tab), 1) - 1
-  body <- body_min:body_max
   top <- tab[1:(body_min - 1)]
   mid <- tab[body_min:body_max]
   mid <- mid[mid != ""]
@@ -38,9 +37,8 @@ group_typst_row <- function(x, i, indent, ...) {
   tab <- paste(tab, collapse = "\n")
   x@table_string <- tab
   idx_new <- i + seq_along(i) - 1
-  idx_all <- seq_len(nrow(x) + length(i))
+  idx_all <- seq_len(nrow(x))
   idx <- setdiff(idx_all, idx_new)
-  x <- style_tt(x, idx, indent = indent)
   return(x)
 }
 
@@ -67,13 +65,5 @@ group_typst_col <- function(x, j, ihead, ...) {
 
   x@table_string <- out
 
-  # # midrule
-  # jrule <- lapply(names(j), function(n) if (trimws(n) != "") j[[n]])
-  # jrule <- Filter(function(k) !is.null(k), jrule)
-  # for (jr in jrule) {
-  #   # 0 indexing
-  #   x <- style_eval(x, i = ihead, j = jr, line = "b", line_width = .05, midrule = TRUE)
-  # }
-
   return(x)
 }
